var _ = require('lodash'),
	mongoose = require('mongoose'),
	Promise = mongoose.Promise,
	Schema = mongoose.Schema,
	ObjectId = require('mongodb').ObjectID;

exports = module.exports = {
	getSyncModel: function(mongoose) {
		"use strict";

		var collectionName = 'DataSource';
		var syncModelName = 'DataSourceSync';

		return (function() {
			var SyncModel;

			if (_.isUndefined(SyncModel)) {

				var dataSourceSyncSchema = new Schema({
					person: {type: Schema.ObjectId, required: true},
					dataSource: {type: String, required: true},
					dataType: {type: String, required: true},
					recordType: {type: String, required: true},
					startDate: {type: Date, required: true},
					endDate: {type: Date, required: true},
					success: {type: Boolean, required: true},
					message: String,
					updatedOn: Date
				}, {
					strict: true
				});

				dataSourceSyncSchema.statics.getCollectionName = function getCollectionName() {
					return collectionName;
				};

				dataSourceSyncSchema.statics.getModelName = function getModelName() {
					return syncModelName;
				};

				dataSourceSyncSchema.statics.getRecordType = function getRecordType() {
					return 'data-source-sync';
				};

				dataSourceSyncSchema.statics.getLatestSuccessfulSync = function getLatestSuccessfulSync(personId, dataSource, dataType) {
					if (_.isString(personId)) { personId = ObjectId.createFromHexString(personId); }

					return SyncModel.find({
						person: personId,
						dataSource: dataSource,
						dataType: dataType,
						recordType: SyncModel.getRecordType(),
						success: true
					})
					.limit(1)
					.sort('-endDate')
					.exec();
				};

				dataSourceSyncSchema.statics.add = function add (personId, startDate, endDate, dataSource, dataType, success, message) {
					if (!_.isDate(startDate)) { startDate = new Date(startDate); }
					if (!_.isDate(endDate)) { endDate = new Date(endDate); }
					if (_.isString(personId)) { personId = ObjectId.createFromHexString(personId); }

					var promise = new Promise();

					var dataSourceDocument = new SyncModel({
						person: personId,
						dataSource: dataSource,
						dataType: dataType,
						recordType: SyncModel.getRecordType(),
						startDate: startDate,
						endDate: endDate,
						success: success,
						message: message,
						active: true
					});

					dataSourceDocument.save(function(error) {
						promise.resolve(error);
					});
					return promise;
				};

				mongoose.connection.collection(collectionName);
				SyncModel = mongoose.model(syncModelName, dataSourceSyncSchema, collectionName);
			}
			return SyncModel;
		}());
	}
};