describe('Data Source', function() {

	var dataSource = require('../lib/data-source'),
		person = require('../lib/person'),
		mongoose = require('mongoose'),
		expect = require('chai').expect,
		moment = require('moment'),
		settings = require('./test-database-configuration');

	var connectionString = 'mongodb://' + settings.databaseServer + ':' + settings.databasePort + '/' + settings.databaseName;

	var DataSourceSyncModel = dataSource.getSyncModel(mongoose);

	afterEach(function(done) {
		mongoose.disconnect(function(){done();});
	});

	beforeEach(function() {
		mongoose.connect(connectionString);
	});

	it('should get the collection name', function() {
		expect(DataSourceSyncModel.getCollectionName()).to.equal('DataSource');
	});

	it('should get the model name', function() {
		expect(DataSourceSyncModel.getModelName()).to.equal('DataSourceSync');
	});

	describe('sync', function() {
		var personId = '517b43ffba2e25e702000005';
		var endDate = moment();
		var startDate = endDate.clone().subtract('minutes', 3);
		var failedEndDate = endDate.clone().add('minutes', 1);
		var failedStartDate = startDate;
		var dataSourceName = 'data-source-test';
		var dataTypeName = 'data-type-test';
		var successMessage = 'data source success test message';
		var failMessage = 'data source fail test message';
		var returnValue, returnError;

		beforeEach(function(done) {
			DataSourceSyncModel.add(personId, startDate, endDate, dataSourceName, dataTypeName, true, successMessage)
				.then(function() {
					return DataSourceSyncModel.add(personId, failedStartDate, failedEndDate, dataSourceName, dataTypeName, false, failMessage);
				})
				.then(function() {
					return DataSourceSyncModel.getLatestSuccessfulSync(personId, dataSourceName, dataTypeName);
				})
				.then(function(item) {
					returnValue = item;
					done();
				},
				function(error) {
					returnError = error;
					done();
				});
		});

		it('should have gotten the latest successful sync', function() {
			expect(returnError, 'Message: ' + (returnError && returnError.message)).to.equal(undefined);

			expect(returnValue).to.not.equal(undefined);
			expect(returnValue.length).to.equal(1);
			expect(returnValue[0].endDate.toISOString()).to.equal(endDate.toDate().toISOString());
		});
	});

});
